@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -24,6 +24,7 @@
 #include "gva-error.h"
 #include "gva-favorites.h"
 #include "gva-mame.h"
+#include "gva-nplayers.h"
 #include "gva-util.h"
 
 #define ASSERT_OK(code) \
@@ -65,6 +66,8 @@
                 "input_tilt DEFAULT 'no' " \
                 "CHECK (input_tilt in ('yes', 'no')), " \
                 "input_players, " \
+                "input_players_alt, " \
+                "input_players_sim, " \
                 "input_buttons, " \
                 "input_coins, " \
                 "driver_status " \
@@ -240,6 +243,8 @@
                 "@input_service, " \
                 "@input_tilt, " \
                 "@input_players, " \
+                "@input_players_alt, " \
+                "@input_players_sim, " \
                 "@input_buttons, " \
                 "@input_coins, " \
                 "@driver_status, " \
@@ -435,6 +440,25 @@ static struct
 
 static sqlite3 *db = NULL;
 
+static void
+db_parser_bind_int (sqlite3_stmt *stmt,
+                    const gchar *param,
+                    gint value)
+{
+        gint index;
+        gint errcode;
+        GError *error = NULL;
+
+        index = sqlite3_bind_parameter_index (stmt, param);
+        errcode = sqlite3_bind_int (stmt, index, value);
+
+        if (errcode != SQLITE_OK)
+        {
+                gva_db_set_error (&error, 0, NULL);
+                gva_error_handle (&error);
+        }
+}
+
 static void
 db_parser_bind_text (sqlite3_stmt *stmt,
                      const gchar *param,
@@ -749,10 +773,13 @@ db_parser_start_element_game (ParserData *data,
                               GError **error)
 {
         sqlite3_stmt *stmt = data->insert_game_stmt;
+        gint ii;
+
+#ifdef CATEGORY_FILE
         gchar *category;
         gint error_code;
-        gint ii;
         GError *local_error = NULL;
+#endif
 
         /* Bind default values. */
         db_parser_bind_text (stmt, "@isbios", "no");
@@ -813,6 +840,13 @@ db_parser_start_element_input (ParserData *data,
         sqlite3_stmt *stmt = data->insert_game_stmt;
         gint ii;
 
+#ifdef NPLAYERS_FILE
+        gint error_code;
+        gint max_alternating = 0;
+        gint max_simultaneous = 0;
+        GError *local_error = NULL;
+#endif
+
         /* Bind default values. */
         db_parser_bind_text (stmt, "@input_service", "no");
         db_parser_bind_text (stmt, "@input_tilt", "no");
@@ -836,6 +870,31 @@ db_parser_start_element_input (ParserData *data,
 
                 db_parser_bind_text (stmt, param, attribute_value[ii]);
         }
+
+#ifdef NPLAYERS_FILE
+        /* Lookup players info from the nplayers.ini file. */
+        g_return_if_fail (data->game != NULL);
+        gva_nplayers_lookup (
+                data->game, &max_alternating,
+                &max_simultaneous, &local_error);
+
+        /* Silence "key not found" errors. */
+        error_code = G_KEY_FILE_ERROR_KEY_NOT_FOUND;
+        if (g_error_matches (local_error, G_KEY_FILE_ERROR, error_code))
+                g_clear_error (&local_error);
+ 
+        db_parser_bind_int (stmt, "@input_players_alt", max_alternating);
+        db_parser_bind_int (stmt, "@input_players_sim", max_simultaneous);
+
+        /* Override "input_players" if we can, because nplayers.ini
+         * seems to be more accurate than MAME's own XML data. */
+        if (max_alternating > 0 || max_simultaneous > 0)
+                db_parser_bind_int (
+                        stmt, "@input_players",
+                        MAX (max_alternating, max_simultaneous));
+        else if (local_error != NULL)
+                g_propagate_error (error, local_error);
+#endif
 }
 
 static void
@@ -1591,9 +1650,9 @@ gva_db_execute (const gchar *sql,
 /**
  * gva_db_get_table:
  * @sql: an SQL statement
- * @result: return location for the result
- * @rows: return location for the number of rows in the result
- * @columns: return location for the number of columns in the result
+ * @result: return location for the result, or %NULL
+ * @rows: return location for the number of rows in the result, or %NULL
+ * @columns: return location for the number of columns in the result, or %NULL
  * @error: return location for a #GError, or %NULL
  *
  * Executes the given SQL statement and returns the results as a string
@@ -1614,25 +1673,28 @@ gva_db_get_table (const gchar *sql,
 {
         char **table;
         char *errmsg;
+        int local_rows = 0;
+        int local_columns = 0;
         gint errcode;
 
         g_return_val_if_fail (db != NULL, FALSE);
         g_return_val_if_fail (sql != NULL, FALSE);
-        g_return_val_if_fail (result != NULL, FALSE);
-        g_return_val_if_fail (rows != NULL, FALSE);
-        g_return_val_if_fail (columns != NULL, FALSE);
 
         errcode = sqlite3_get_table (
-                db, sql, &table, rows, columns, &errmsg);
+                db, sql, &table, &local_rows, &local_columns, &errmsg);
 
         if (errcode == SQLITE_OK)
         {
-                gint length, ii;
+                if (result != NULL)
+                {
+                        gint length, ii;
+
+                        length = (local_rows + 1) * (local_columns);
+                        *result = g_new0 (gchar *, length + 1);
+                        for (ii = 0; ii < length; ii++)
+                                (*result)[ii] = g_strdup (table[ii]);
+                }
 
-                length = (*rows + 1) * (*columns);
-                *result = g_new0 (gchar *, length + 1);
-                for (ii = 0; ii < length; ii++)
-                        (*result)[ii] = g_strdup (table[ii]);
                 sqlite3_free_table (table);
         }
         else
@@ -1641,6 +1703,11 @@ gva_db_get_table (const gchar *sql,
                 sqlite3_free (errmsg);
         }
 
+        if (rows != NULL)
+                *rows = local_rows;
+        if (columns != NULL)
+                *columns = local_columns;
+
         return (errcode == SQLITE_OK);
 }
 
