@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2009 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -187,7 +187,7 @@ gva_main_init (void)
         gtk_entry_set_text (GTK_ENTRY (GVA_WIDGET_MAIN_SEARCH_ENTRY), text);
         g_free (text);
 
-#ifndef WITH_GNOME
+#if !GTK_CHECK_VERSION(2,14,0) && !defined WITH_GNOME
         /* Requires that we link against libgnome. */
         gtk_action_set_sensitive (GVA_ACTION_CONTENTS, FALSE);
 #endif
